#include "ndestylegenericdata.h"
#include <QEvent>
#include <QTextStream>
#include <QDebug>
namespace NdeStyle
{

GenericData::GenericData(QObject* parent, QWidget* target):
    QObject( parent), m_target(target){

}
GenericData::~GenericData( void ){

}
void GenericData::addRipple(MaterialRipple *ripple){

    m_animations.push_back(ripple);
    ripple->start();

    connect(ripple, SIGNAL(destroyRipple()), this, SLOT(removeRipple()));
    connect(this, SIGNAL(destroyed(QObject*)), ripple, SLOT(stop()));
    connect(this, SIGNAL(destroyed(QObject*)), ripple, SLOT(deleteLater()));
}
void GenericData::removeRipple(){
    MaterialRipple *ripple = qobject_cast<MaterialRipple *>(sender());
    if(m_animations.removeOne(ripple)){
        ripple->deleteLater();
    }
}

void GenericData::removeAllRipple(){
    foreach(MaterialRipple *ripple, m_animations){
        m_animations.removeOne(ripple);
        ripple->deleteLater();
    }
}

QWidget * GenericData::target() const{
    return m_target;
}
QList<RippleData> GenericData::ripples(){
    QList<RippleData> result;
    foreach(MaterialRipple *ripple, m_animations){
        RippleData data;
        data.radius = ripple->radius();
        data.opacity = ripple->opacity();
        data.center = ripple->center();
        result.append(data);
    }
    return result;
}

GenericWidthData::GenericWidthData(QObject* parent, QWidget* target):
    QObject( parent), m_widthTarget(target){

}
GenericWidthData::~GenericWidthData( void ){

}
void GenericWidthData::addWidthRipple(MaterialWidthRipple *ripple){

    m_widthAnimations.push_back(ripple);
    ripple->start();

    connect(ripple, SIGNAL(destroyRipple()), this, SLOT(removeRipple()));
    connect(this, SIGNAL(destroyed(QObject*)), ripple, SLOT(stop()));
    connect(this, SIGNAL(destroyed(QObject*)), ripple, SLOT(deleteLater()));
}
void GenericWidthData::removeRipple(){
    MaterialWidthRipple *ripple = qobject_cast<MaterialWidthRipple *>(sender());
    if(m_widthAnimations.removeOne(ripple)){
        ripple->deleteLater();
    }
}

void GenericWidthData::removeAllRipple(){
    foreach(MaterialWidthRipple *ripple, m_widthAnimations){
        m_widthAnimations.removeOne(ripple);
        ripple->deleteLater();
    }
}

QWidget * GenericWidthData::target() const{
    return m_widthTarget;
}



QList<RippleWidthData> GenericWidthData::widthRipples(){
    QList<RippleWidthData> result;
    foreach(MaterialWidthRipple *ripple, m_widthAnimations){
        RippleWidthData data;

        data.width = ripple->width();
        result.append(data);
    }
    return result;
}



}
