#include "ndestylematerialripple.h"
#include <QDebug>

namespace NdeStyle
{
MaterialRipple::MaterialRipple(const QPoint &center, QWidget *target, QObject *parent)
    : QParallelAnimationGroup(parent),
      m_radiusAnimation(animate("radius")),
      m_opacityAnimation(animate("opacity")),
      m_target(target),
      m_radius(0),
      m_opacity(0),
      m_center(center)
{
    if (target){
        connect(target, SIGNAL(destroyed()), this, SLOT(destroyWidget()));
        connect(target, SIGNAL(destroyed()), this, SIGNAL(destroyRipple()));
        connect(target, SIGNAL(destroyed()), this, SIGNAL(destroyed()));
    }

    if (!(target->inherits("QSlider") || target->inherits("QTabBar"))){
        init();
    }

}

MaterialRipple::~MaterialRipple()
{
}

void MaterialRipple::destroyWidget()
{
    this->stop();
    this->clear();
}

void MaterialRipple::setRadius(qreal radius)
{
    if (qFuzzyCompare(m_radius, radius)) {
        return;
    }
    m_radius = radius;
    m_target->update();
}

void MaterialRipple::setOpacity(qreal opacity)
{
    if (qFuzzyCompare(m_opacity,opacity)) {
        return;
    }
    m_opacity = opacity;
    m_target->update();
}

QPropertyAnimation *MaterialRipple::animate(const QByteArray &property,
                                              const QEasingCurve &easing,
                                              int duration)
{
    QPropertyAnimation *animation = new QPropertyAnimation;
    animation->setTargetObject(this);
    animation->setPropertyName(property);
    animation->setEasingCurve(easing);
    animation->setDuration(duration);
    addAnimation(animation);
    return animation;
}

void MaterialRipple::init()
{
    setOpacityStartValue(0.5);
    setOpacityEndValue(0);
    setRadiusStartValue(0);
    setRadiusEndValue(300);

    connect(this, SIGNAL(finished()), this, SIGNAL(destroyRipple()));
}

MaterialWidthRipple::MaterialWidthRipple(const qreal &lineWidth, QWidget *target, QObject *parent)
    : QParallelAnimationGroup(parent),
      m_widthAnimation(animate("width")),
      m_target(target),
      m_width(0)
{
    if (target){
        connect(target, SIGNAL(destroyed()), this, SLOT(destroyWidget()));
        connect(target, SIGNAL(destroyed()), this, SIGNAL(destroyRipple()));
        connect(target, SIGNAL(destroyed()), this, SIGNAL(destroyed()));
    }

    init(lineWidth);
}

MaterialWidthRipple::~MaterialWidthRipple()
{
}

void MaterialWidthRipple::destroyWidget()
{
    this->stop();
    this->clear();
}

void MaterialWidthRipple::setWidth(qreal width)
{
    if (qFuzzyCompare(m_width,width)) {
        return;
    }
    m_width = width;
    m_target->update();
}

QPropertyAnimation *MaterialWidthRipple::animate(const QByteArray &property,
                                              const QEasingCurve &easing,
                                              int duration)
{
    QPropertyAnimation *animation = new QPropertyAnimation;
    animation->setTargetObject(this);
    animation->setPropertyName(property);
    animation->setEasingCurve(easing);
    animation->setDuration(duration);
    addAnimation(animation);
    return animation;
}

void MaterialWidthRipple::init(const qreal width)
{
    setWidthStartValue(0);
    setWidthEndValue(width);
}

}
