#ifndef ndestylematerialripple_h
#define ndestylematerialripple_h

#include <QParallelAnimationGroup>
#include <QEasingCurve>
#include <QPropertyAnimation>
#include <QPoint>
#include <QWidget>
namespace NdeStyle
{
class MaterialRipple : public QParallelAnimationGroup
{
    Q_OBJECT

    Q_PROPERTY(qreal radius WRITE setRadius READ radius)
    Q_PROPERTY(qreal opacity WRITE setOpacity READ opacity)

public:
    explicit MaterialRipple(const QPoint &center, QWidget* target, QObject *parent = nullptr);
    ~MaterialRipple();

    void setRadius(qreal radius);
    inline qreal radius() const;

    void setOpacity(qreal opacity);
    inline qreal opacity() const;

    inline QPoint center() const;

    inline QPropertyAnimation *radiusAnimation() const;
    inline QPropertyAnimation *opacityAnimation() const;

    inline void setOpacityStartValue(qreal value);
    inline void setOpacityEndValue(qreal value);
    inline void setRadiusStartValue(qreal value);
    inline void setRadiusEndValue(qreal value);

    inline void setDuration(int msecs);

Q_SIGNALS:
    void destroyRipple();

protected Q_SLOTS:
    void destroyWidget();

private:
    Q_DISABLE_COPY(MaterialRipple)

    QPropertyAnimation *animate(const QByteArray &property,
                                const QEasingCurve &easing = QEasingCurve::OutQuad,
                                int duration = 800);

    void init();

    QPropertyAnimation *const m_radiusAnimation;
    QPropertyAnimation *const m_opacityAnimation;
    QWidget            *const m_target;
    qreal                     m_radius;
    qreal                     m_opacity;
    QPoint                    m_center;
};
inline qreal MaterialRipple::radius() const
{
    return m_radius;
}

inline qreal MaterialRipple::opacity() const
{
    return m_opacity;
}
inline QPoint MaterialRipple::center() const
{
    return m_center;
}


inline QPropertyAnimation *MaterialRipple::radiusAnimation() const
{
    return m_radiusAnimation;
}

inline QPropertyAnimation *MaterialRipple::opacityAnimation() const
{
    return m_opacityAnimation;
}

inline void MaterialRipple::setOpacityStartValue(qreal value)
{
    m_opacityAnimation->setStartValue(value);
}

inline void MaterialRipple::setOpacityEndValue(qreal value)
{
    m_opacityAnimation->setEndValue(value);
}

inline void MaterialRipple::setRadiusStartValue(qreal value)
{
    m_radiusAnimation->setStartValue(value);
}

inline void MaterialRipple::setRadiusEndValue(qreal value)
{
    m_radiusAnimation->setEndValue(value);
}


inline void MaterialRipple::setDuration(int msecs)
{
    m_radiusAnimation->setDuration(msecs);
    m_opacityAnimation->setDuration(msecs);
}


class MaterialWidthRipple : public QParallelAnimationGroup
{
    Q_OBJECT

    Q_PROPERTY(qreal width WRITE setWidth READ width)

public:
    explicit MaterialWidthRipple(const qreal &lineWidth, QWidget* target, QObject *parent = nullptr);
    ~MaterialWidthRipple();

    void setWidth(qreal width);
    inline qreal width() const;
    inline QWidget* target() const;

    inline QPropertyAnimation *widthAnimation() const;

    inline void setWidthStartValue(qreal value);
    inline void setWidthEndValue(qreal value);
    inline void setDuration(int msecs);

    qreal LineWidth;

Q_SIGNALS:
    void destroyRipple();

protected Q_SLOTS:
    void destroyWidget();

private:
    Q_DISABLE_COPY(MaterialWidthRipple)

    QPropertyAnimation *animate(const QByteArray &property,
                                const QEasingCurve &easing = QEasingCurve::OutQuad,
                                int duration = 800);

    void init(const qreal width);

    QPropertyAnimation *const m_widthAnimation;
    QWidget            *const m_target;
    qreal                     m_width;
};

inline qreal MaterialWidthRipple::width() const
{
    return m_width;
}

inline QWidget* MaterialWidthRipple::target() const
{
    return m_target;
}

inline QPropertyAnimation *MaterialWidthRipple::widthAnimation() const
{
    return m_widthAnimation;
}

inline void MaterialWidthRipple::setWidthStartValue(qreal value)
{
    m_widthAnimation->setStartValue(value);
}

inline void MaterialWidthRipple::setWidthEndValue(qreal value)
{
    m_widthAnimation->setEndValue(value);
}

inline void MaterialWidthRipple::setDuration(int msecs)
{
    m_widthAnimation->setDuration(msecs);
}
}
#endif
